"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
const express = require("express");
const bodyParser = require("body-parser");
const session = require("express-session");
const path = require("path");
const _types_1 = require("./@types");
const ProjectMetadata_1 = require("./metadata/ProjectMetadata");
exports.default = new class KatrineApp {
    constructor() {
        this.httpPort = 3400;
        this.sessionConfig = {
            secret: "gfw4sdffgw4tas34532sdfg",
            resave: false,
            saveUninitialized: true
        };
        this.express = express();
        this.express.use(bodyParser.urlencoded({ extended: false }));
        this.express.use(bodyParser.json());
        this.express.use(session(this.sessionConfig));
    }
    setPublicFolder(folder) {
        this.express.use(express.static(folder));
    }
    loadControllers(controllers) {
        controllers.forEach(contPath => {
            let basePath = '';
            if (contPath[0] === '.') {
                basePath = path.dirname(require.main.filename) + '/';
            }
            const controller = require(basePath + contPath);
            if (!controller) {
                throw 'Controller is undefined';
            }
        });
    }
    initActions() {
        ProjectMetadata_1.default.getActions().forEach((action) => {
            const boundAction = ProjectMetadata_1.default.getActionByRoute(action.getRoute());
            switch (action.getRequestType()) {
                case _types_1.HTTPRequestType.GET:
                    this.express.get(action.getRoute(), this.requestHandler.bind(this, boundAction));
                    break;
                case _types_1.HTTPRequestType.POST:
                    this.express.post(action.getRoute(), this.requestHandler.bind(this, boundAction));
                    break;
                default:
                    throw new Error(`Can't handle "${action.getActionMethodName()}" HTTP method.`);
            }
        });
        this.express.all('*', this.handle404.bind(this));
    }
    handle404(req, res) {
        const action = ProjectMetadata_1.default.getActionByRoute(_types_1.HTTPStatusCode.PAGE_NOT_FOUND);
        if (action) {
            this.callAction(action, req, res, _types_1.HTTPStatusCode.PAGE_NOT_FOUND);
            return;
        }
        res.status(_types_1.HTTPStatusCode.PAGE_NOT_FOUND);
        res.send(JSON.stringify({
            'status': 'error',
            'description': 'Page not found...'
        }));
    }
    handle403(req, res) {
        const action = ProjectMetadata_1.default.getActionByRoute(_types_1.HTTPStatusCode.ACCESS_FORBIDDEN);
        if (action) {
            this.callAction(action, req, res, _types_1.HTTPStatusCode.ACCESS_FORBIDDEN);
            return;
        }
        res.status(_types_1.HTTPStatusCode.ACCESS_FORBIDDEN);
        res.send(JSON.stringify({
            'status': 'error',
            'description': 'Access forbidden...'
        }));
    }
    getActionPromise(action, req, res) {
        return new Promise((resolve, reject) => {
            try {
                const responce = action.invoke(req, res);
                if (typeof responce == 'string') {
                    resolve(responce);
                }
                else if (responce && (responce.constructor.name === 'Promise' ||
                    responce.constructor.name === 'WrappedPromise')) {
                    responce.then((responceString) => {
                        resolve(responceString);
                    });
                }
                else {
                    const baseError = `Unsupported type returned from Controller "${typeof responce}"
            Please retrun "string" or "Promise" from your actions.
            https://www.npmjs.com/package/katrine#controllers-and-actions`;
                    let constructorType = '';
                    if (responce && responce.constructor && responce.constructor.name) {
                        constructorType = `Instance of wrong object "${responce.constructor.name}"`;
                    }
                    reject(baseError + constructorType);
                }
            }
            catch (e) {
                reject(e);
            }
        });
    }
    callAction(action, req, res, status = _types_1.HTTPStatusCode.SUCCESS) {
        const accessError = action.canAccess(req);
        if (accessError) {
            this.handle403(req, res);
            return;
        }
        this.getActionPromise(action, req, res)
            .then((respString) => {
            res.status(status);
            res.send(respString);
        })
            .catch((error) => {
            console.error(error);
            res.status(500);
            res.send('Internal server error...');
        });
    }
    requestHandler(action, req, res) {
        // because express send by fourth parameter next() callback, need to wrap
        this.callAction(action, req, res);
    }
    applyConfig(config) {
        if (!(config && typeof config == 'object')) {
            return;
        }
        if (config.httpServer) {
            if (config.httpServer.port) {
                this.httpPort = config.httpServer.port;
            }
            if (config.httpServer.sessionConfig) {
                Object.assign(this.sessionConfig, config.httpServer.sessionConfig);
            }
        }
    }
    run(config) {
        this.applyConfig(config);
        if (!ProjectMetadata_1.default.hasAnyActions()) {
            this.initActions();
        }
        if (!ProjectMetadata_1.default.hasAnyActions()) {
            throw new Error('Application must contain at least 1 action');
        }
        this.express.listen(this.httpPort, (err) => {
            if (err) {
                console.error(err);
                return;
            }
            console.log(`server is listening on ${this.httpPort}`);
            return;
        });
    }
    auth(session, user) {
        session.authenticated = true;
        session.userData = user.getUserData();
        session.uid = user.getId();
        session.role = user.getRole();
    }
    authLogoutUser(session) {
        session.authenticated = false;
        session.userData = {};
        session.uid = null;
        session.role = null;
    }
};
