"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
const _types_1 = require("../@types");
const AccessByRole_1 = require("../rbac/AccessByRole");
const AccessByAuth_1 = require("../rbac/AccessByAuth");
/**
 * Central element for Initialization and build Katrine components (controllers, actions, access rules e.t.c).
 */
class ProjectMetadata {
    constructor() {
        this._actions = new Map();
        this._rawControllers = new Map();
        this._controllerInstances = new Map();
        this._boundedActions = new Map();
        this._accessRules = [];
    }
    getActions() {
        return Array.from(this._actions).map(item => item[1]);
    }
    hasAnyActions() {
        return !!this._boundedActions.size;
    }
    getActionByRoute(route) {
        if (!this._actions.has(route)) {
            throw `Action for route ${route} does not exist`;
        }
        if (!this._boundedActions.has(route)) {
            this.compileAction(route);
        }
        return this._boundedActions.get(route);
    }
    storeAction(action) {
        this._actions.set(action.getRoute(), action);
        // this._controllers.set(action.getControllerName(), action.getController())
    }
    addAccessByRoleRule(ruleData, actionName, controller) {
        this._accessRules.push(Object.assign({ action: this.makeKeyByController(actionName, controller) }, ruleData));
    }
    addController(controllerName, controller) {
        this._rawControllers.set(controllerName, controller);
    }
    compileAction(route) {
        const action = this._actions.get(route);
        let controllerInstance = action.getController();
        if (!controllerInstance) {
            controllerInstance = this._controllerInstances.get(action.getControllerName());
            if (!controllerInstance) {
                const controllerClass = action.getControllerClass();
                controllerInstance = new controllerClass.constructor();
                this._controllerInstances.set(action.getControllerName(), controllerInstance);
            }
        }
        const boundAction = controllerInstance[action.getActionMethodName()].bind(controllerInstance);
        action.setController(controllerInstance);
        action.setInvoker(boundAction);
        this._boundedActions.set(route, action);
        // access control
        this.applyAccessRulesToAction(action);
    }
    applyAccessRulesToAction(action) {
        const rules = this.findAccessRulesForAction(action);
        if (!rules.length) {
            return;
        }
        let _rule;
        rules.forEach((rule) => {
            switch (rule.type) {
                case _types_1.AccessRulesType.ByRole:
                    _rule = new AccessByRole_1.default(rule);
                    break;
                case _types_1.AccessRulesType.ByAuth:
                    _rule = new AccessByAuth_1.default(rule);
                    break;
                default:
                    throw `Rule ${rule.type} is not supported`;
            }
            if (_rule) {
                action.addAccessRule(_rule);
            }
        });
    }
    findAccessRulesForAction(action) {
        const actionKey = this.makeKeyByController(action.getActionMethodName(), action.getControllerClass());
        const rules = this._accessRules.filter((rule) => rule.action === actionKey);
        return rules;
    }
    makeKeyByController(actionName, controller) {
        if (!controller.constructor || !controller.constructor.name) {
            throw `Constructor or constructor name is undefined. It can't be use for identification`;
        }
        return `${controller.constructor.name}_${actionName}`;
    }
}
exports.default = new ProjectMetadata();
